from excelipy.models import Style

DEFAULT_HEADER_STYLE = Style(
    bold=True,
    background="#ecedef",
    border=1,
    border_color="#9fa3ac",
)

DEFAULT_BODY_STYLE = Style(
    border=1,
    border_color="#9fa3ac",
)
